var spamNumber = Math.floor(Math.random() * (100 - 1 + 1)) + 1;
$(function(){
    $('.submit-btn').click(function(event){
        //Prevent form submission
        event.preventDefault();
        var inputs = $('input');
        for (var i = 0; i < inputs.length; i++) {
            var input = inputs[i];
            doValidation(input);
        }
        if ($('.error-data').length == 0) {
            //No errors, submit the form
            $('#webForm').submit();
        }
    });
    $('input').on("keyup", function(){
        doValidation($(this));
    });
    $('.password').on("keyup", function(){
        var score = 0;
        var password = $('.password');
        var passwordAgain = $('.confirm-password');
        //Remove any old errors for the password fields
        password.parent().removeClass('error');
        password.next('.error-data').remove();
        passwordAgain.parent().removeClass('error');
        passwordAgain.next('.error-data').remove();
            //Password is greater than 6 characters
            if (password.val().length > 6) {
                score++;
            }
            //Password is greater than 8 characters
            if (password.val().length > 8) {
                score++;
            }
            //Password has both uppercase and lowercase characters
            if (/(?=.*[A-Z])(?=.*[a-z])/.test(password.val())) {
                score++;
            }
            //Password has at least one number
            if (/(?=.*[0-9])/.test(password.val())) {
                score++;
            }
            //Password has at least one symbol (@$!&^) character
            if (/@|\$|\!|&|\^/.test(password.val())) {
                score++;
            }
            var fill = (100 - ((score * 2) * 10));
            var percent = (100 - fill);
            var level,
                colour;
            switch (score) {
                case 0:
                case 1:
                    level = "Weak";
                    colour = "green";
                    break;
                case 2:
                case 3:
                    level = "Medium";
                    colour = "orange";
                    break;
                case 4:
                    level = "Strong";
                    colour = "red";
                    break;
                case 5:
                    level = "Excellent";
                    colour = "purple";
                    break;
            }
            $('.password-strength .inner').css('right', fill + "%").css('background-color', colour);
            $('.password-strength .text').html(level + " (" + percent + "%)");
    });
    $('.anti-spam-number').html(spamNumber);
});

function doValidation(input) {
    //Remove old errors
    $(input).parent().removeClass('error');
    $(input).next('.error-data').remove();
    if ($(input).hasClass('required') && !validateRequired($(input).val())) {
        addErrorData($(input), "This is a required field");
    }
    if ($(input).hasClass('email') && !validateEmail($($(input)).val())) {
        addErrorData($(input), "Invalid email address provided");
    }
    if ($(input).hasClass('number') && !validateNumber($(input).val())) {
        addErrorData($(input), "This field can only contain numbers");
    }
    if ($(input).hasClass('date') && !validateDate($(input).val())) {
        addErrorData($(input), "Invalid date provided");
    }
    if ($(input).hasClass('credit-card') && !validateCreditCard($(input).val())) {
        addErrorData($(input), "Invalid credit card number");
    }
    if ($(input).hasClass('confirm-password')) {
        var result = validatePasswords($(input).val());
        if (result != true) {
            addErrorData($(input), result);
        }
    }
    if ($(input).hasClass('anti-spam-input') && !validateAntiSpam($(input).val())) {
        addErrorData($(input), "Incorrect Anti-Spam answer");
    }
}

function validateRequired(value) {
    if (value == "") return false;
    return true;
}
function validateEmail(value) {
    if (value != "") {
        return /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/i.test(value);
    }
    return true;
}
function validateCreditCard(value) {
    if (value != "") {
        return /^(?:4[0-9]{12}(?:[0-9]{3})?|5[1-5][0-9]{14}|6(?:011|5[0-9][0-9])[0-9]{12}|3[47][0-9]{13}|3(?:0[0-5]|[68][0-9])[0-9]{11}|(?:2131|1800|35\d{3})\d{11})$/.test(value);
    }
    return true;
}
function validateNumber(value) {
    if (value != "") {
        return !isNaN(parseInt(value, 10)) && isFinite(value);
        //isFinite, in case letter is on the end
    }
    return true;
}
function validateDate(value) {
    if (value != "") {
        if (/^\d{2}([.\/-])\d{2}\1\d{4}$/.test(value)) {
            // Remove leading zeros
            value = value.replace(/0*(\d*)/gi,"$1");
            var dateValues = value.split(/[\.|\/|-]/);
            // Correct the month value as month index starts at 0 now 1 (e.g. 0 = Jan, 1 = Feb)
            dateValues[1]--;
            var date = new Date(dateValues[2], dateValues[1], dateValues[0]);
            if (
                date.getDate() == dateValues[0] &&
                    date.getMonth() == dateValues[1] &&
                    date.getFullYear() == dateValues[2]
                ) {
                return true;
            }
        }
        return false;
    } else {
        return true;
    }
}

function validatePasswords(value) {
    var password = $('.password').val();
    if (value == "") {
        return "Both passwords are required";
    } else if (value != password) {
        return "Passwords do not match";
    }
    return true;
}

function validateAntiSpam(value) {
    if (value != "") {
        if (parseInt(value)!= spamNumber) return false;
    }
    return true;
}

function addErrorData(element, error) {
    element.parent().addClass("error");
    element.after("<div class='error-data'>" + error + "</div>");
}